@@ -118,22 +118,12 @@ load_thumb_done_cb (ThumbLoader *tl,
 }
 
 
-static void
-free_file_data_list (GList *list)
-{
-	GList *scan;
-	for (scan = list; scan; scan = scan->next) 
-		file_data_unref (scan->data);
-	g_list_free (list);
-}
-
-
 static void
 gth_file_list_free_list (GthFileList *file_list)
 {
 	g_return_if_fail (file_list != NULL);
 
-	free_file_data_list (file_list->list);
+	file_data_list_free (file_list->list);
 	file_list->list = NULL;
 }
 
@@ -401,12 +391,7 @@ get_file_info_data_free (GetFileInfoData *data)
 		g_list_free (data->uri_list);
 	}
 
-	if (data->filtered != NULL) {
-		g_list_foreach (data->filtered, 
-				(GFunc) file_data_unref, 
-				NULL);
-		g_list_free (data->filtered);
-	}
+	file_data_list_free (data->filtered);
 
 	g_free (data);
 }
@@ -432,8 +417,6 @@ set_list__get_file_info_done_cb (GnomeVFSAsyncHandle *handle,
 	if (file_list->interrupt_set_list) {
 		DoneFunc done_func;
 
-		g_print ("[1]\n");
-
 		file_list->interrupt_set_list = FALSE;
 		done_func = file_list->interrupt_done_func;
 		file_list->interrupt_done_func = NULL;
@@ -449,7 +432,7 @@ set_list__get_file_info_done_cb (GnomeVFSAsyncHandle *handle,
 		char                      *escaped;
 		FileData                  *fd;
 
-		if (info_result->result != GNOME_VFS_OK) 
+		if ((info_result->result != GNOME_VFS_OK) || (info_result->uri == NULL))
 			continue;
 
 		escaped = gnome_vfs_uri_to_string (info_result->uri, GNOME_VFS_URI_HIDE_TOPLEVEL_METHOD);
@@ -475,8 +458,6 @@ set_list__step2 (GetFileInfoData *gfi_data)
 	if (file_list->interrupt_set_list) {
 		DoneFunc done_func;
 
-		g_print ("[2]\n");
-
 		g_signal_emit (G_OBJECT (file_list), gth_file_list_signals[IDLE], 0);
 
 		file_list->interrupt_set_list = FALSE;
@@ -558,8 +539,6 @@ gth_file_list_interrupt_set_list (GthFileList *file_list,
 {
 	g_return_if_fail (file_list != NULL);
 
-	g_print ("[0]\n");
-
 	if (file_list->interrupt_set_list) {
 		if (done_func != NULL)
 			(*done_func) (done_data);
@@ -591,8 +570,6 @@ add_list_in_chunks (gpointer callback_data)
 	if (file_list->interrupt_set_list) {
 		DoneFunc  done_func;
 
-		g_print ("[3]\n");
-
 		file_list->enable_thumbs = gfi_data->enable_thumbs;
 
 		file_list->interrupt_set_list = FALSE;
@@ -684,8 +661,6 @@ add_list__get_file_info_done_cb (GnomeVFSAsyncHandle *handle,
 	if (file_list->interrupt_set_list) {
 		DoneFunc done_func;
 	
-		g_print ("[4]\n");
-
 		done_func = file_list->interrupt_done_func;
 		file_list->interrupt_done_func = NULL;
 		if (done_func != NULL)
@@ -723,13 +698,11 @@ void
 add_list__step2 (GetFileInfoData *gfi_data)
 {
 	GnomeVFSAsyncHandle *handle;
-	GthFileList            *file_list = gfi_data->file_list;
+	GthFileList         *file_list = gfi_data->file_list;
 
 	if (file_list->interrupt_set_list) {
 		DoneFunc done_func;
 
-		g_print ("[5]\n");
-
 		done_func = file_list->interrupt_done_func;
 		file_list->interrupt_done_func = NULL;
 		if (done_func != NULL)
@@ -1048,7 +1021,7 @@ gth_file_list_get_selection_length (GthFileList *file_list)
 
 	sel_list = gth_file_view_get_selection (file_list->view);
 	len = g_list_length (sel_list);
-	g_list_free (sel_list);
+	file_data_list_free (sel_list);
 
 	return len;
 }
@@ -1059,6 +1032,7 @@ gth_file_list_path_from_pos (GthFileList *file_list,
 			     int          pos)
 {
 	FileData *fd;
+	char     *retval = NULL;
 
 	g_return_val_if_fail (file_list != NULL, NULL);
 
@@ -1066,11 +1040,11 @@ gth_file_list_path_from_pos (GthFileList *file_list,
 		return NULL;
 
 	fd = gth_file_view_get_image_data (file_list->view, pos);
-
 	if ((fd != NULL) && (fd->path != NULL))
-		return g_strdup (fd->path);
+		retval = g_strdup (fd->path);
+	file_data_unref (fd);
 
-	return NULL;
+	return retval;
 }
 
 
@@ -1198,8 +1172,11 @@ gth_file_list_next_image (GthFileList *file_list,
 
 		fd = gth_file_view_get_image_data (file_list->view, pos);
 
-		if (without_error && fd->error)
+		if (without_error && fd->error) {
+			file_data_unref (fd);
 			continue;
+		}
+		file_data_unref (fd);
 
 		if (only_selected && ! gth_file_view_pos_is_selected (file_list->view, pos))
 			continue;
@@ -1228,8 +1205,11 @@ gth_file_list_prev_image (GthFileList *file_list,
 
 		fd = gth_file_view_get_image_data (file_list->view, pos);
 
-		if (without_error && fd->error)
+		if (without_error && fd->error) {
+			file_data_unref (fd);
 			continue;
+		}
+		file_data_unref (fd);
 
 		if (only_selected && ! gth_file_view_pos_is_selected (file_list->view, pos))
 			continue;
@@ -1258,6 +1238,7 @@ delete_pos__step2 (InterruptThumbsData *it_data)
 
 	g_return_if_fail (fd != NULL);
 
+	file_data_unref (fd);
 	file_list->list = g_list_remove (file_list->list, fd);
 	file_data_unref (fd);
 
@@ -1313,6 +1294,7 @@ rename_pos__step2 (InterruptThumbsData *it_data)
 
 	/* Set the new name. */
 	gth_file_view_set_image_text (file_list->view, pos, fd->utf8_name);
+	file_data_unref (fd);
 
 	gth_file_view_sorted (file_list->view,
 			      file_list->sort_method,
@@ -1367,6 +1349,7 @@ update_comment__step2 (InterruptThumbsData *it_data)
 
 	/* Set the new name. */
 	gth_file_view_set_image_comment (file_list->view, pos, fd->comment);
+	file_data_unref (fd);
 
 	if (it_data->restart_thumbs) {
 		file_list->doing_thumbs = TRUE;
@@ -1528,7 +1511,8 @@ gth_file_list_update_thumb (GthFileList  *file_list,
 
 	file_list->thumb_pos = pos;
 	file_list->thumb_fd = fd;
-	
+	file_data_unref (fd);
+
 	gth_file_list_update_current_thumb (file_list);
 }
 
@@ -1537,7 +1521,7 @@ void
 gth_file_list_update_thumb_list (GthFileList  *file_list,
 				 GList        *list)
 {
-	GList        *scan;
+	GList *scan;
 
 	if (! file_list->enable_thumbs)
 		return;
@@ -1553,10 +1537,10 @@ gth_file_list_update_thumb_list (GthFileList  *file_list,
 			continue;
 
 		fd = gth_file_view_get_image_data (file_list->view, pos);
-
 		file_data_update (fd);
 		fd->error = FALSE;
 		fd->thumb = FALSE;
+		file_data_unref (fd);
 	}
 	
 	file_list->doing_thumbs = TRUE;
@@ -1582,11 +1566,11 @@ gth_file_list_thumb_cleanup (GthFileList *file_list)
 static void
 gth_file_list_update_next_thumb (GthFileList *file_list)
 {
-	FileData     *fd = NULL;
-	int           first_pos, last_pos;
-	int           pos = -1;
-	GList        *list, *scan;
-	int           new_pos = -1;
+	FileData *fd = NULL;
+	int       first_pos, last_pos;
+	int       pos = -1;
+	GList    *list, *scan;
+	int       new_pos = -1;
 
 	if (! file_list->doing_thumbs) {
 		interrupt_thumbs__part2 (file_list);
