@@ -325,7 +325,7 @@ view_result_cb (GtkWidget  *widget,
 {
 	GThumbWindow *window = data->window;
 	Catalog      *catalog;
-	gchar        *catalog_name, *catalog_path;
+	char         *catalog_name, *catalog_path, *catalog_name_utf8;
 	GList        *scan;
 	GError       *gerror;
 
@@ -334,11 +334,13 @@ view_result_cb (GtkWidget  *widget,
 
 	catalog = catalog_new ();
 
-	catalog_name = g_strconcat (_("Search Result"),
-				    CATALOG_EXT,
-				    NULL);
+	catalog_name_utf8 = g_strconcat (_("Search Result"),
+					 CATALOG_EXT,
+					 NULL);
+	catalog_name = g_locale_from_utf8 (catalog_name_utf8, -1, 0, 0, 0);
 	catalog_path = get_catalog_full_path (catalog_name);
 	g_free (catalog_name);
+	g_free (catalog_name_utf8);
 
 	catalog_set_path (catalog, catalog_path);
 	catalog_set_search_data (catalog, data->search_data);
@@ -1097,7 +1099,7 @@ uri_from_comment_uri (const char *comment_uri,
 
 	comment_uri_l = strlen (comment_uri);
 
-	base = comments_get_comment_dir (NULL, TRUE);
+	base = comments_get_comment_dir (NULL, TRUE, TRUE);
 	base_l = strlen (base);
 
 	if (comment_uri_l == base_l) 
@@ -1259,7 +1261,7 @@ search_dir_async (DialogData *data, char *dir)
 	char *start_from;
 
 	if (data->search_comments) 
-		start_from = comments_get_comment_dir (dir, TRUE);
+		start_from = comments_get_comment_dir (dir, TRUE, TRUE);
 	else
 		start_from = g_strdup (dir);
 
