@@ -24,18 +24,20 @@
 #define FULLSCREEN_H
 
 #include "image-viewer.h"
-#include "window.h"
+#include "gthumb-window.h"
 
 typedef struct {
 	GtkWidget     *window;
 	GtkWidget     *viewer;
+	GtkWidget     *toolbar;
 	GThumbWindow  *related_win;
 	guint          motion_id;
 	guint          mouse_hide_id;
+	gboolean       wm_state_fullscreen_support;
 } FullScreen;
 
 
-FullScreen *   fullscreen_new    ();
+FullScreen *   fullscreen_new    (void);
 
 void           fullscreen_close  (FullScreen *fullscreen);
 
